import React, { PureComponent } from 'react';
import { Modal } from 'antd';
import styles from './index.less';

const confirm = Modal.confirm;

/**
 * 转换日期
 * eg: 80000 => 8; 200000 => 20; 201522 => 20
 * @param  {String} time
 * @return {Number}      返回整点 数字类型
 */
function transTime(timeParam = '') {
  const time = timeParam.startsWith('0') ? timeParam.substring(1) : timeParam;
  const subIndex = time.length === 5 ? 1 : 2;

  return Number(`${time}`.substring(0, subIndex));
}

/**
 * 转换时间列表
 * [0,1,2,21,22] => [0,1,2,21,22]; [{startTime:'000000', endTime: '20000‘}, {startTime:'210000', endTime: '220000‘}] => [0,1,2,21,22]
 * @param  {Array} list
 * @return {Array}      返回存在的时间段
 */
function transTimeLimit(list = []) {
  if (list.length === 0 || Number.isInteger(list[0])) return list;

  if (typeof list[0] === 'object') {
    const newList = [];
    const rangeList = [];

    list.forEach(({ startTime = '', endTime = '' }, i) => {
      const st = transTime(startTime);
      const end = transTime(endTime);
      const pushList = [st, end];
      let insertIndex = rangeList.findIndex(item => end < item[0]);

      if (insertIndex === -1) {
        rangeList.push(pushList);
      } else {
        rangeList.splice(insertIndex, 0, pushList);
      }
    });

    for (let i = 0; i < rangeList[rangeList.length - 1][1]; i++) {
      if (rangeList.some(item => i >= item[0] && i < item[1])) {
        newList.push(i);
      }
    }

    return newList;
  }

  return list;
}

function getUpdateList(st, count, mainValueList) {
  const list = [];

  for (let i = st; i < count; i++) {
    list.push({
      text: i,
      isSelect: mainValueList.includes(i),
      isPreSelect: false,
    });
  }

  return list;
}

export default class BlockSelect extends PureComponent {
  constructor() {
    super();

    this.changeBlockStyle = function() {
      let width = this.selectRow.offsetWidth;
      let { list } = this.state;

      let singleWidth = Math.floor((width - 12 - list.length - 1) / list.length);

      this.setState({
        blockStyle: {
          width: singleWidth,
          height: singleWidth,
          pTop: singleWidth + 10,
        },
        wrapperStyle: {
          height: singleWidth + 15,
        },
      });
    }.bind(this);
  }

  state = {
    list: [],
    lastList: [],
    isInit: false,
    wrapperStyle: {
      height: 70,
    },
    blockStyle: {},
    lastSelectIndex: 0,
    clickSwitch: false,
    value: [],
    isControl: false,
  };

  static getDerivedStateFromProps(props, state) {
    if (!state.isInit) {
      const { count = 24, st = 0, value, defalutValue = [] } = props;

      let mainValueList = [];
      let isControl = false;

      if (value && Array.isArray(value)) {
        mainValueList = transTimeLimit(value);
        isControl = true;
      } else {
        mainValueList = transTimeLimit(defalutValue);
      }

      const list = getUpdateList(st, count, mainValueList);

      return {
        list,
        lastList: list,
        isInit: true,
        isControl,
      };
    }

    if (props.value !== state.value && state.isControl) {
      const { count = 24, st = 0 } = props;
      let mainValueList = transTimeLimit(props.value);
      const list = getUpdateList(st, count, mainValueList);

      return {
        list,
        lastList: list,
        value: props.value,
      };
    }

    return null;
  }

  componentDidMount() {
    this.changeBlockStyle();

    window.addEventListener('resize', this.changeBlockStyle);
  }

  render() {
    const { list, blockStyle, wrapperStyle } = this.state;

    return (
      <div
        className={styles.blockSelect}
        style={wrapperStyle}
        ref={ref => {
          this.selectRow = ref;
        }}
      >
        {list.map((item, index) => (
          <div
            className={this.getBlockClass(item)}
            key={index}
            style={{ width: blockStyle.width, height: blockStyle.height }}
            onClick={() => this.openSwitch(index)}
            onMouseEnter={() => this.selectBlock(index)}
          >
            <p style={{ transform: `translate3d(-5px, ${blockStyle.pTop}px, 0)` }}>{item.text}</p>
            {index === list.length - 1 ? (
              <p
                className={styles.lastText}
                style={{ transform: `translate3d(5px, ${blockStyle.pTop}px, 0)` }}
              >
                {list[list.length - 1].text + 1}
              </p>
            ) : null}
          </div>
        ))}
      </div>
    );
  }

  getBlockClass = item => {
    let addClass = '';
    if (item.isPreSelect) {
      addClass = ` ${styles.preActive}`;
    }

    if (item.isSelect) {
      addClass = ` ${styles.active}`;
    }

    return `${styles.blocks}${addClass}`;
  };

  getBlockState = lastItem => {
    if (!lastItem.isSelect) {
      return {
        ...lastItem,
        isPreSelect: true,
      };
    }

    return lastItem;
  };

  getCloseBlockState = item => {
    if (item.isPreSelect) {
      return {
        ...item,
        isSelect: true,
        isPreSelect: false,
      };
    }

    return item;
  };

  openSwitch = index => {
    const { clickSwitch, list } = this.state;

    if (!clickSwitch) {
      let newList = list.slice();
      const lastItem = list[index];

      if (!lastItem.isSelect) {
        newList[index] = this.getBlockState(lastItem);

        this.setState({
          list: newList,
          clickSwitch: true,
          lastList: list,
          lastSelectIndex: index,
        });
      } else {
        this.cancelListRang(index);
      }
    } else {
      const newList = list.map((item, i) => this.getCloseBlockState(item));
      const params = {
        clickSwitch: false,
      };

      if (!this.state.isControl) {
        params.list = newList;
        params.lastList = newList;
      } else {
        params.list = this.state.lastList;
      }

      this.setState(params);

      this.handlerChangeEvent(newList);
    }
  };

  selectBlock = index => {
    const { clickSwitch, list, lastList, lastSelectIndex } = this.state;
    const rangeIndex = [index, lastSelectIndex].sort((a, b) => a - b);

    if (!clickSwitch) return;

    const newList = list.map((item, i) => {
      if (i >= rangeIndex[0] && i <= rangeIndex[1]) {
        return this.getBlockState(lastList[i]);
      }

      return lastList[i];
    });

    this.setState({
      list: newList,
    });
  };

  cancelListRang = index => {
    confirm({
      title: '确定是否需要清空该时间段',
      okText: '确定',
      cancelText: '取消',
      onOk: () => {
        const { lastList } = this.state;
        const newList = new Array(lastList.length);
        let ltSwitch = true;
        let rgSwitch = true;

        for (let i = index; i >= 0; i--) {
          if (lastList[i].isSelect && ltSwitch) {
            newList[i] = {
              ...lastList[i],
              isSelect: false,
            };
          } else {
            newList[i] = {
              ...lastList[i],
            };
            ltSwitch = false;
          }
        }

        for (let i = index; i < lastList.length; i++) {
          if (lastList[i].isSelect && rgSwitch) {
            newList[i] = {
              ...lastList[i],
              isSelect: false,
            };
          } else {
            newList[i] = {
              ...lastList[i],
            };
            rgSwitch = false;
          }
        }

        const params = {};

        if (!this.state.isControl) {
          params.list = newList;
          params.lastList = newList;
        } else {
          params.list = this.state.lastList;
        }

        this.setState(params);

        this.handlerChangeEvent(newList);
      },
    });
  };

  handlerChangeEvent = newList => {
    const { onChange } = this.props;
    const rangeList = [];
    const timeLimitList = [];

    let selectSwitch = false;
    let lastSwitch = false;
    let lastIndex = -1;
    let differenceCount = 0;

    newList.forEach((item, i) => {
      if (item.isSelect) rangeList.push(i);
      if (i === 0) {
        if (item.isSelect) {
          differenceCount++;
          lastIndex = 0;
        }
      } else if (item.isSelect !== newList[i - 1].isSelect) {
        if (differenceCount % 2) {
          timeLimitList.push({
            startTime: `${lastIndex < 10 ? '0' : ''}${lastIndex}${'0'.repeat(4)}`,
            endTime: `${i < 10 ? '0' : ''}${i}${'0'.repeat(4)}`,
          });
        } else {
          lastIndex = i;
        }

        differenceCount++;
      }

      if (i === newList.length - 1 && item.isSelect) {
        timeLimitList.push({
          startTime: `${lastIndex < 10 ? '0' : ''}${lastIndex}${'0'.repeat(4)}`,
          endTime: `${i < 10 ? '0' : ''}${i + 1}${'0'.repeat(4)}`,
        });
      }
    });

    onChange && onChange(rangeList, timeLimitList);
  };
}
