import React, { PureComponent } from 'react';
import { Input, Select, Icon, AutoComplete, message } from 'antd';
import request from '@/utils/request';
import { debounce } from '@/utils/decorator';
import styles from './HotelSearch.less';

const Option = Select.Option;

/**
 * 用于酒店
 * @extends PureComponent
 */
class HotelSearch extends PureComponent {
  constructor() {
    super();
    this.state = {
      type: '0',
      idInput: '',
      nameInput: '',
      selected: {}, // 已被选择的
      searchList: [],
      isFixedType: false,
    };
  }

  componentWillMount() {
    let { defaultValue, type } = this.props;

    // if (!defaultValue) return;

    let newState = {
      nameInput: defaultValue,
      selected: {
        hotel_name: defaultValue,
      },
    };

    type = `${type}`;

    if (type === '0' || type === '1') {
      newState.type = type;
      newState.isFixedType = true;
    }

    this.setState(newState);
  }

  componentWillReceiveProps(nextProps) {
    if (this.props.cityId !== nextProps.cityId && nextProps.cityId) {
      const { onChange } = nextProps;

      this.setState({
        selected: {},
        searchList: [],
        nameInput: '',
      });

      onChange && onChange({});
    }
  }

  render() {
    const { type, idInput, searchList, nameInput, isFixedType } = this.state;

    return (
      <div
        className={`${styles.searchGroup} ${type === '0' ? styles.searchName : styles.searchId}`}
      >
        {!isFixedType && (
          <Select defaultValue="0" style={{ width: 160 }} onChange={val => this.onSearchState(val)}>
            <Option value="0">酒店名称查询</Option>
            <Option value="1">酒店ID查询</Option>
          </Select>
        )}
        {this.state.type === '0' ? (
          <AutoComplete
            dataSource={searchList.map(item => ({
              value: item.id,
              text: item.hotel_name,
            }))}
            style={{ marginTop: 3 }}
            value={nameInput}
            onFocus={() => this.requestSearch()}
            onSearch={val => this.handleSearchName(val)}
            onSelect={val => this.handlerSelect(val)}
            onBlur={val => this.handleSearchBlur(val)}
            className={isFixedType ? styles.round : styles.halfRound}
            placeholder="请输入酒店名称"
            ref={ref => {
              this.AutoComplete = ref;
            }}
          />
        ) : (
          <Input
            placeholder="请输入酒店ID"
            value={idInput}
            onChange={e => this.onSearchId(e)}
            key="id"
            ref={ref => {
              this.idInput = ref;
            }}
          />
        )}
        {this.state.type === '0' ? null : (
          <div className={styles.btn} onClick={() => this.handlerIdSearch()}>
            <Icon type="search" theme="outlined" />
          </div>
        )}
      </div>
    );
  }

  /**
   * 搜索酒店接口
   * @param  {[type]}  params [description]
   * @return {Promise}        [description]
   */
  async suggestHotel(params) {
    try {
      const response = await request(`/api/crs/v1/common/suggestHotel`, {
        method: 'POST',
        body: {
          ...params,
        },
      });

      return Promise.resolve({
        statue: true,
        ...response,
      });
    } catch (err) {
      return Promise.resolve({
        statue: false,
        ...err.response,
      });
    }
  }

  handlerSelect(val) {
    const { onChange } = this.props;
    const item = this.getSelectItem(val); // 获取已经选择的

    if (item) {
      this.setState({
        selected: item,
        nameInput: item.hotel_name,
      });

      onChange && onChange(item);
    }
  }

  // 酒店名字搜索(绑定debounce避免过多请求)
  handleSearchName(val) {
    console.log(val);
    this.setState({
      nameInput: val,
    });

    this.requestSearch(val);
  }

  @debounce(1000)
  async requestSearch(val = this.state.nameInput) {
    const { onChange, cityId } = this.props;
    const payload = {
      hotelName: val,
      searchType: 'hotelNameSearch',
    };

    // 传入城市id的时候增加城市id的搜索
    if (cityId) {
      payload.cityId = cityId;
    }

    // 全数字是因为AutoComplete组件选择了 option下拉框
    const res = await this.suggestHotel(payload); // 获取酒店数据

    if (res.statue && res.data && res.data.hotels) {
      this.setState({
        searchList: res.data.hotels, // 设置搜索表单
      });
    } else {
      this.setState({
        searchList: [],
      });
    }
  }

  onSearchState(val) {
    if (val === this.state.type) return;

    const { onChange } = this.props;

    this.setState({
      selected: {},
      type: val,
      idInput: '',
    });

    onChange && onChange({});
  }

  handleSearchBlur(val) {
    const { onChange } = this.props;
    const item = this.getSelectItem(val);

    if (item) {
      this.setState({
        selected: item,
      });

      onChange && onChange(item);
    }
  }

  // 根据变化的值
  getSelectItem(val) {
    const { searchList, selected } = this.state;

    // 全数字是因为AutoComplete组件选择了 option下拉框
    if (/^\d+$/.test(val)) {
      return searchList.find(item => item.id === val);
    }
    if (selected.hotel_name === val) return false;

    return searchList.find(item => item.hotel_name === val);
  }

  async handlerIdSearch() {
    const { idInput } = this.state;
    const { onChange } = this.props;
    const payload = {
      hotelIds: [idInput],
      searchType: 'hotelIdSearch',
    };

    const res = await this.suggestHotel(payload); // 获取酒店数据

    if (res.statue && res.data) {
      const item = res.data.hotels;

      if (item && item.length) {
        this.setState({
          selected: item[0],
        });
        message.success(`搜索到酒店${item[0].hotel_name}`);
        onChange && onChange(item[0]);
      }
    } else {
      this.setState({
        selected: {},
      });
      onChange && onChange({});
      message.error('不存在该id的酒店');
    }
  }

  onSearchId(e) {
    const val = e.target.value;

    if (Number.isNaN(Number(val))) return false;

    this.setState({
      idInput: val,
    });
  }

  focus() {
    if (this.state.type === '0') {
      this.AutoComplete.focus();
    } else {
      this.idInput.focus();
    }
  }
}

export default HotelSearch;
