import React, { PureComponent } from 'react';
import { connect } from 'dva';
import { AutoComplete, Button, Input, Select, Icon, message, Radio } from 'antd';
import Bind from 'lodash-decorators/bind';
import Debounce from 'lodash-decorators/debounce';
import Throttle from 'lodash-decorators/throttle';
import BaiduMapSearch from '@/components/BaiduMapSearch';
import styles from './SearchPart.less';

const InputGroup = Input.Group;
const Option = Select.Option;
const labelStyle = { display: 'inline-block', textAlign: 'right', minWidth: 100 };
const RadioGroup = Radio.Group;

const radioStyle = {
  display: 'flex',
  alignItems: 'center',
  height: '30px',
  lineHeight: '30px',
};

/**
 * 输入时搜索下拉列表，选中时将获取到的经纬度保存到state，
 * 点击搜索时，将经纬度提交到dva，再执行外部进来的搜索方法
 */
@connect(({ nearby }) => ({ nearby }))
export default class SearchPart extends PureComponent {
  state = {
    searchType: 'hotelName',
    searchValue: '',
    namePos: {
      latitude: undefined,
      longitude: undefined,
    },
    baiduPos: {
      latitude: undefined,
      longitude: undefined,
    },
    searchIndexType: 1,
  };

  // 选择搜索类型
  onSelectChange = searchType => {
    this.setState({
      searchType,
      searchValue: '',
      namePos: {
        latitude: undefined,
        longitude: undefined,
      },
    });
  };

  // 选中列表的某个项
  onValueSelected = position => {
    // model中将经纬度放在数组中并序列化作为option的value，所以这里需要反序列化
    const [latitude, longitude] = JSON.parse(position);

    this.setState({
      namePos: {
        latitude,
        longitude,
      },
      searchIndexType: 1,
    });
  };

  onValueChange = searchValue => {
    this.setState({ searchValue }, () => {
      // 空串的时候清空下拉列表
      if (!searchValue) {
        this.props.dispatch({ type: 'nearby/setHotelList', payload: { hotelList: [] } });
      } else if (!searchValue.startsWith('[')) {
        // 多一步判断[，防止select的时候再查询，
        this.doSearch(searchValue);
      }
    });
  };

  @Bind()
  @Debounce(750)
  doSearch(searchValue) {
    return this.props.dispatch({
      type: 'nearby/searchHotel',
      payload: {
        start: 0,
        hits: 500,
        // 属性名不能直接放this.state.searchType，ES6可以放在括号中计算属性名
        [this.state.searchType]: searchValue,
      },
    });
  }

  /**
   * 保存地标搜索的经纬度到dva
   */
  onMapSearch = ({ lat: latitude, lng: longitude }) => {
    this.setState({
      baiduPos: {
        latitude,
        longitude,
      },
      searchIndexType: 2,
    });
  };

  /**
   * 判断输入框不为空，提交坐标到dva，调用外部fn，都为空则提示无推荐，并清空表格
   *
   */
  onSearch = () => {
    const { namePos, baiduPos, searchIndexType } = this.state;
    let latitude = null;
    let longitude = null;

    if (searchIndexType === 1) {
      ({ latitude, longitude } = namePos);
    } else {
      ({ latitude, longitude } = baiduPos);
    }

    if (!latitude || !longitude) {
      message.info('5km内无任何可推荐的酒店，请检查或更换搜索条件');
      this.props.dispatch({ type: 'nearby/setNearbyHotels', payload: { nearbyHotels: [] } });
      return;
    }

    const { onSearch } = this.props;

    if (onSearch && typeof onSearch === 'function') {
      onSearch({
        latitude,
        longitude,
      });
    }
  };

  /**
   * 重置，情况两个输入框，以及经纬度
   */
  onReset = () => {
    this.setState({
      searchValue: '',
      baiduPos: {
        latitude: undefined,
        longitude: undefined,
      },
      namePos: {
        latitude: undefined,
        longitude: undefined,
      },
      searchIndexType: 1,
    });
    this.baiduMapSearch.reset();
    this.props.dispatch({
      type: 'nearby/setHotelList',
      payload: { hotelList: [] },
    });
  };

  handlerSearchIndexType = val => {
    this.setState({
      searchIndexType: val,
    });
  };

  render() {
    const { hotelList } = this.props.nearby;
    const { searchType, searchIndexType, searchValue, baiduPos } = this.state;

    return (
      <div style={{ minWidth: 900 }} className={styles.searchPart}>
        <RadioGroup
          onChange={e => this.handlerSearchIndexType(e.target.value)}
          value={searchIndexType}
        >
          <Radio style={radioStyle} value={1}>
            <div onClick={() => this.handlerSearchIndexType(1)}>
              <Select
                value={searchType}
                onChange={this.onSelectChange}
                className={styles.selectType}
                style={{ width: 100, marginLeft: 20 }}
              >
                <Option value="hotelName">酒店名称</Option>
                <Option value="oyoId">OYO ID</Option>
              </Select>
              <AutoComplete
                style={{ width: 800 }}
                value={searchValue}
                dataSource={hotelList}
                onChange={this.onValueChange}
                onSelect={this.onValueSelected}
                className={styles.autoComplete}
                placeholder={searchType === 'hotelName' ? '请输入酒店名称' : '请输入OYO ID'}
              />
            </div>
          </Radio>
          <Radio style={{ display: 'flex', alignItems: 'center', marginTop: 30 }} value={2}>
            <div onClick={() => this.handlerSearchIndexType(2)}>
              <span style={{ paddingLeft: 10, paddingRight: 58, marginLeft: 20 }}>地标:</span>
              <BaiduMapSearch
                ref={ref => {
                  this.baiduMapSearch = ref;
                }}
                onChange={this.onMapSearch}
                placeholder="交通枢纽 / 地铁站 / 商圈 / 地标"
                style={{ width: 800 }}
              />
              <span style={{ paddingLeft: 20, fontSize: 20 }}>
                {searchIndexType === 2 && baiduPos.latitude && baiduPos.longitude ? (
                  <Icon type="check-circle" />
                ) : null}
              </span>
            </div>
          </Radio>
        </RadioGroup>

        <div style={{ marginTop: 20, textAlign: 'center' }}>
          <Button onClick={this.onSearch} type="primary">
            搜索
          </Button>
          <Button onClick={this.onReset} style={{ marginLeft: 20 }}>
            重置
          </Button>
        </div>
      </div>
    );
  }
}
