import React, { PureComponent } from 'react';
import { EditorState, convertToRaw } from 'draft-js';
import moment from 'moment';
import { Editor } from 'react-draft-wysiwyg';
import draftToHtml from 'draftjs-to-html';
import { Card, Row, Col, Input, Button, Modal, Form, DatePicker, message } from 'antd';
import PageHeaderWrapper from '@/components/PageHeaderWrapper';
import OYOTable from '@/components/OyoTable/OyoNestedTable';
import { queryNoticeList, getNoticeDetail, addNotice } from './services/noticeList';
import ButtonAuthorized from '@/components/ButtonAuthorized';
// import styles from './react-draft-wysiwyg.css';
import styles from './noticeList.less';

const FormItem = Form.Item;
const { TextArea } = Input;
const { RangePicker } = DatePicker;

const startTimeFormat = 'YYYY-MM-DD 00:00:00';
const endTimeFormat = 'YYYY-MM-DD 23:59:59';
const currentDay = moment();
const oneMonthBefore = moment().subtract(30, 'days');

export default class NoticeList extends PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      pageParams: ['pageNum', 'pageSize'],
      // 表格数据
      tableParams: [],
      // 弹窗是否展示
      modalVisible: false,
      // 弹窗类型
      modalType: '',
      // 公告信息
      noticeDetailInfo: {
        title: '',
        content: '',
        sendTime: '',
        expireTime: '',
      },
      // 搜索框-标题
      title: '',
      // 搜索时间
      searchTime: {
        startSendTime: oneMonthBefore.format(startTimeFormat),
        endSendTime: currentDay.format(endTimeFormat),
      },
      // 时间
      searchTimeValue: [],
      // 表格配置项
      tablePagination: {
        current: 1,
        pageSize: 10,
        total: 0,
      },
      editorState: EditorState.createEmpty(),
      modalTitle: '通知公告',
    };
    // 表格配置头.
    this.OYOTableColumns = [
      {
        title: '通知公告标题',
        dataIndex: 'title',
        width: 200,
      },
      {
        title: '发送时间',
        dataIndex: 'sendTime',
        width: 200,
      },
      {
        title: '结束时间',
        dataIndex: 'expireTime',
        width: 200,
      },
      {
        title: '操作',
        width: 200,
        dataIndex: 'action',
        render: (action, data) => (
          <ButtonAuthorized
            code="CRS13.1.2"
            size="small"
            onClick={() => this.handleModalVisible(data.id)}
          >
            详 情
          </ButtonAuthorized>
        ),
      },
    ];
  }

  componentDidMount() {
    this.getNoticeList();
  }

  // 获取通告公告的列表
  getNoticeList = ({ pageNum } = { pageNum: 1 }) => {
    const {
      searchTime: { startSendTime, endSendTime },
      tablePagination: { current },
      title = '',
    } = this.state;
    const params = {
      title,
      startSendTime,
      endSendTime,
      pageNum: pageNum || 1,
    };
    queryNoticeList(params)
      .then(res => {
        if (!Array.isArray(res.data.result)) {
          return;
        }
        const {
          data: { result, totalCount },
        } = res;
        this.setState({
          tableParams: result,
          tablePagination: {
            ...this.state.tablePagination,
            total: totalCount,
            current: pageNum,
          },
        });
      })
      .catch(err => {
        console.log(err);
      });
  };

  // 获取公告详情
  queryNoticeDetail = id => {
    const params = {
      id,
    };
    getNoticeDetail(params)
      .then(res => {
        if (!res.data) {
          return;
        }
        const {
          data: { title, content, sendTime, expireTime },
        } = res;
        this.setState({
          noticeDetailInfo: {
            title,
            content,
            sendTime,
            expireTime,
          },
        });
      })
      .catch(err => {
        console.log(err);
      });
  };

  // 搜索框数据改变
  handleChangeParams = (ev, key) => {
    this.setState({
      [key]: ev.target.value ? ev.target.value : null,
    });
  };

  // 点击重置按钮.输入框重置为空/日期在充值为前30天到今天.
  handleResetBtn = () => {
    this.setState(
      {
        title: '',
        searchTimeValue: [],
        searchTime: {
          startSendTime: oneMonthBefore.format(startTimeFormat),
          endSendTime: currentDay.format(endTimeFormat),
        },
      },
      () => this.getNoticeList()
    );
  };

  // 点击增加按钮
  handleClickAddBtn = () => {
    this.setState({
      modalVisible: true,
      modalType: 'add',
      modalTitle: '新增通知公告',
    });
  };

  // 点击表格中的详情按钮
  handleModalVisible = id => {
    this.setState({
      modalVisible: true,
      modalType: 'info',
      modalTitle: '通知公告详情',
    });
    this.queryNoticeDetail(id);
  };

  // 点击弹窗取消按钮
  handleModalCancelBtn = () => {
    this.setState({
      modalVisible: false,
      editorState: EditorState.createEmpty(),
      noticeDetailInfo: {
        title: '',
        content: '',
        sendTime: '',
        expireTime: '',
      },
    });
  };

  // 点击弹窗确认按钮.详情时直接取消/增加时调取接口
  handleModalSubmitBtn = () => {
    const { modalType } = this.state;
    if (modalType === 'info') {
      this.handleModalCancelBtn();
    } else if (modalType === 'add') {
      const {
        noticeDetailInfo: { title, content, expireTime },
      } = this.state;
      if (!title || !content || !expireTime) {
        message.error('请输入正确的公告信息');
        return;
      }
      this.handleAddNotice();
    }
  };

  // 增加公告.
  handleAddNotice = () => {
    const {
      noticeDetailInfo: { title, content, expireTime },
    } = this.state;
    const params = {
      title,
      content,
      expireTime,
    };
    addNotice(params)
      .then(res => {
        this.setState({
          editorState: EditorState.createEmpty(),
          modalVisible: false,
          noticeDetailInfo: {
            title: '',
            content: '',
            sendTime: '',
            expireTime: '',
          },
        });
        this.getNoticeList();
      })
      .catch(err => {
        console.log(err);
      });
  };

  // 时间选择器选择.
  onChangeRangePicker = (date, dateString) => {
    const { searchTime } = this.state;
    this.setState({
      searchTimeValue: date,
      searchTime: {
        startSendTime: moment(dateString[0]).format(startTimeFormat),
        endSendTime: moment(dateString[1]).format(endTimeFormat),
      },
    });
  };

  // 改变增加公告的值.
  handleChangeAddNotice = (ev, key) => {
    this.setState({
      noticeDetailInfo: {
        ...this.state.noticeDetailInfo,
        [key]: ev.target.value ? ev.target.value : null,
      },
    });
  };

  // 结束时间获取.
  onChangeEndTimePicker = (date, dateString) => {
    this.setState({
      noticeDetailInfo: {
        ...this.state.noticeDetailInfo,
        expireTime: dateString,
      },
    });
  };

  // 设置发送时间的不可点击范围
  setSendTimeDisabledDate = value => {
    if (!value) {
      return false;
    }
    return value < oneMonthBefore || value > moment();
  };

  // 设置ExpireTime的不可点击范围
  setExpireTimePicker = value => {
    return value < moment();
  };

  // 富文本编辑器的change事件.
  onEditorStateChange = editorState => {
    const {
      noticeDetailInfo: { content },
    } = this.state;
    this.setState({
      editorState,
      noticeDetailInfo: {
        ...this.state.noticeDetailInfo,
        content: draftToHtml(convertToRaw(editorState.getCurrentContent())),
      },
    });
  };

  render() {
    const {
      pageParams,
      tableParams,
      modalVisible,
      modalType,
      noticeDetailInfo: { title, content, sendTime, expireTime },
      tablePagination,
      editorState,
      modalTitle,
    } = this.state;
    return (
      <PageHeaderWrapper>
        <Card>
          {/* 上侧查询区域 */}
          <Row>
            {/* 标题 */}
            <Col span={8}>
              <Input
                value={this.state.title}
                placeholder="请输入通知公告标题"
                onChange={v => this.handleChangeParams(v, 'title')}
              />
            </Col>
            {/* 发送日期 */}
            <Col span={8} style={{ marginLeft: 30 }}>
              <RangePicker
                allowClear={false}
                disabledDate={this.setSendTimeDisabledDate}
                value={this.state.searchTimeValue}
                onChange={this.onChangeRangePicker}
              />
            </Col>
            {/* 查询与重置 */}
            <Col span={6} style={{ marginLeft: 30 }}>
              <Button type="primary" onClick={this.getNoticeList}>
                查询
              </Button>
              <Button onClick={this.handleResetBtn} style={{ marginLeft: 20 }}>
                重置
              </Button>
            </Col>
          </Row>
          {/* 新增按钮 */}
          <Row style={{ marginTop: 30 }}>
            <Col span={24}>
              <ButtonAuthorized
                code="CRS13.1.1"
                icon="plus"
                type="primary"
                onClick={() => this.handleClickAddBtn()}
              >
                新增
              </ButtonAuthorized>
            </Col>
          </Row>
          {/* 表格 */}
          <OYOTable
            rowKey="id"
            bordered
            size="small"
            dataSource={tableParams}
            style={{ marginTop: 20 }}
            getLogList={this.getNoticeList}
            columns={this.OYOTableColumns}
            pageParams={pageParams}
            pagination={tablePagination}
          />
        </Card>
        {/* 弹窗 */}
        <Modal
          title={modalTitle}
          okText="确认"
          width={700}
          destroyOnClose
          maskClosable={false}
          onOk={() => this.handleModalSubmitBtn()}
          onCancel={() => this.handleModalCancelBtn()}
          visible={modalVisible}
        >
          {/* 新增/详情-标题 */}
          <FormItem labelCol={{ span: 5 }} wrapperCol={{ span: 15 }} label="标题">
            <Input
              placeholder="不超过40个字符"
              disabled={modalType === 'info'}
              value={title}
              maxLength={40}
              onChange={v => this.handleChangeAddNotice(v, 'title')}
            />
          </FormItem>
          {/* 新增/详情-内容 */}
          <FormItem labelCol={{ span: 5 }} wrapperCol={{ span: 15 }} label="内容">
            {modalType === 'info' ? (
              <div className={styles.noticeInfo} dangerouslySetInnerHTML={{ __html: content }} />
            ) : (
              <div style={{ height: '400px', border: '1px solid #d9d9d9', maxHeight: '400px' }}>
                <Editor
                  placeholder="字数限制：1000字"
                  editorState={editorState}
                  localization={{ locale: 'zh' }}
                  wrapperClassName="contract-template-add-wrapper"
                  editorClassName="contract-template-editor"
                  editorStyle={{ backgroundColor: '', height: '286px', maxHeight: '286px' }}
                  onEditorStateChange={this.onEditorStateChange}
                  toolbar={{
                    options: [
                      'inline',
                      'blockType',
                      'fontSize',
                      'fontFamily',
                      'textAlign',
                      'colorPicker',
                      'remove',
                      'history',
                    ],
                  }}
                />
              </div>
            )}
          </FormItem>
          {/* 详情-发送时间 */}
          {modalType === 'info' ? (
            <FormItem labelCol={{ span: 5 }} wrapperCol={{ span: 15 }} label="发送时间">
              <Input disabled={modalType === 'info'} placeholder="发送时间" value={sendTime} />
            </FormItem>
          ) : null}
          {/* 新增/详情-结束时间 */}
          <FormItem labelCol={{ span: 5 }} wrapperCol={{ span: 15 }} label="结束时间">
            {modalType === 'info' ? (
              <Input disabled placeholder="结束时间" value={expireTime} />
            ) : (
              <DatePicker
                showTime
                placeholder="请选择结束时间的时分秒"
                allowClear={false}
                style={{ width: '100%' }}
                onChange={this.onChangeEndTimePicker}
                format="YYYY-MM-DD HH:mm:ss"
                disabledDate={this.setExpireTimePicker}
              />
            )}
          </FormItem>
        </Modal>
      </PageHeaderWrapper>
    );
  }
}
