import React, { PureComponent } from 'react';
import { Table, Divider, Dropdown, Button, Menu, Icon, Tag, Empty, Card } from 'antd';
import { connect } from 'dva';
import moment from 'moment';
import router from 'umi/router';
import Styles from './Bill.less';

const columns = [
  {
    title: '房间号',
    dataIndex: 'roomNo',
    key: 'roomNo',
    width: '9%',
  },
  {
    title: '营业日',
    dataIndex: 'bizDate',
    key: 'bizDate',
    width: '13%',
    render: (bizDate, record) => (bizDate ? moment(bizDate).format('YYYY-MM-DD') : ''),
  },
  {
    title: '项目名称',
    dataIndex: 'name',
    key: 'name',
    width: '11%',
    render: (bizDate, record) =>
      `${record.name}${record.payTypeName ? `(${record.payTypeName})` : ''}`,
  },
  {
    title: '结算',
    key: 'settlementFee',
    dataIndex: 'settlementFee',
    width: '9%',
    render: (text, record) => (record.type === 1 ? record.amount : '-'),
  },
  {
    title: '消费',
    key: 'consumptionFee',
    dataIndex: 'consumptionFee',
    width: '9%',
    render: (text, record) => (record.type !== 1 ? record.amount : '-'),
  },
  {
    title: '余额',
    key: 'remaining',
    dataIndex: 'remaining',
    width: '9%',
  },
  {
    title: '凭证号',
    key: 'voucherNumber',
    dataIndex: 'voucherNumber',
    width: '17%',
  },
  {
    title: '操作时间',
    key: 'createTime',
    dataIndex: 'createTime',
    width: '13%',
  },
  {
    title: '摘要',
    key: 'showRemark',
    dataIndex: 'showRemark',
    width: '10%',
  },
];

@connect(({ global, order, user, loading }) => ({
  order,
  user,
  global,
  loading: loading.models.order,
}))
export default class BillDetail extends PureComponent {
  state = {
    bookingData: {},
    orderStatusColor: ['#f50', '#87d068', '#2db7f5', '', ''],
    page: {
      pageSize: 20,
      current: 1,
      total: 50,
      showSizeChanger: true,
    },
  };

  componentWillMount() {
    const {
      location,
      dispatch,
      order: { roomReservationData },
    } = this.props;
    const bookingId = location.query.bookingId || '';

    if (bookingId === '') {
      router.push('/order/list');
      return;
    }
    // Room和Bill两块只调用一次服务器数据
    // if (roomReservationData !== null && roomReservationData.length !== 0) {
    //   return;
    // }
    dispatch({ type: 'order/fetchRoomReservationList', payload: { bookingId } })
      .then(data => {
        if (data.code === '000') {
          if (data.data && data.data.length !== 0) {
            this.setState({
              bookingData: data.data[0],
            });
            this.getHouseDataList(data.data[0].orderId, data.data[0].orderSn);
          }
        }
      })
      .catch(e => {
        console.log(e);
      });
  }

  // 账单基本消费信息
  renderRemainingAmount = (room, key) => {
    if (room.remainingAmount === null) {
      return (
        <p className={Styles.BillTotal} key={`renderRemainingAmount-${key}`}>
          <span>{`消费合计: ${room.postingSum.consumptionAmount || 0}元`}</span>
          <span>{`结算合计: ${room.postingSum.payAmount || 0}元`}</span>
          <span key={`renderRemainingAmount-${key}`} />
        </p>
      );
    }

    return (
      <p className={Styles.BillTotal} key={`renderRemainingAmount-${key}`}>
        <span>{`消费合计: ${room.postingSum.consumptionAmount || 0}元`}</span>
        <span>{`结算合计: ${room.postingSum.payAmount || 0}元`}</span>
        <span>{`余额: ${room.postingSum.remainingAmount || 0}元`}</span>
      </p>
    );
  };

  handleMenuClick = e => {
    const {
      order: { roomReservationData },
    } = this.props;
    const { page } = this.state;
    this.setState({
      page: {
        ...page,
        current: 1,
        pageSize: 20,
        total: 0,
      },
    });
    if (e) {
      this.setState({
        bookingData: roomReservationData[e.key],
      });
    }
    this.getHouseDataList(roomReservationData[e.key].orderId, roomReservationData[e.key].orderSn);
  };

  // 获取房单的数据
  getHouseDataList = (id, orderSn) => {
    const { page } = this.state;
    const { dispatch } = this.props;
    dispatch({
      type: 'order/fetchRoomDataList',
      payload: {
        orderSn,
        orderId: id,
        page: page.current,
        limit: page.pageSize,
      },
    })
      .then(data => {
        if (data.code === '000' && data.data) {
          this.setState({
            page: {
              ...page,
              total: data.data.totalCount || 20,
            },
          });
        }
      })
      .catch(e => {
        console.log(e);
      });
  };

  // 房单切换
  renderDropDown = () => {
    const { bookingData } = this.state;
    const {
      order: { roomReservationData },
    } = this.props;

    const menu = (
      <Menu onClick={this.handleMenuClick}>
        {roomReservationData.map((room, key) => (
          <Menu.Item key={key}>
            {room.isGroup === 1 && room.unionFlag === 1
              ? `团队主帐：${room.groupName}`
              : room.orderSn}
          </Menu.Item>
        ))}
      </Menu>
    );
    return (
      <Dropdown trigger={['click']} overlay={menu}>
        <Button style={{ marginLeft: 8 }}>
          {bookingData.isGroup === 1 && bookingData.unionFlag === 1
            ? `团队主帐：${bookingData.groupName}`
            : bookingData.orderSn || roomReservationData[0].orderSn || 'loading...'}{' '}
          <Icon type="down" />
        </Button>
      </Dropdown>
    );
  };

  // 表格分页pageSize和PageNum回调
  handlePageChange = pagination => {
    const { page, bookingData } = this.state;
    const { dispatch } = this.props;
    this.setState({
      page: {
        ...page,
        current: pagination.current,
        pageSize: pagination.pageSize,
      },
    });
    dispatch({
      type: 'order/fetchRoomDataList',
      payload: {
        orderId: bookingData.orderId,
        page: pagination.current,
        limit: pagination.pageSize,
      },
    });
  };

  renderEmptyBill = () => (
    <Card>
      <Empty />
    </Card>
  );

  renderNotEmptyBill = () => {
    const {
      global,
      order: { roomReservationData, houseDataList },
    } = this.props;
    const { bookingData, page, orderStatusColor } = this.state;
    const defaultBooking = roomReservationData[0];
    return (
      <div className={Styles.BillPage}>
        <div className={Styles.BillPageBlock}>
          <div className={Styles.BillPageHd}>
            {this.renderDropDown()}
            <p style={{ marginTop: 6, marginLeft: 20 }} className={Styles.BillCode}>
              房单号: <span>{bookingData.orderSn || defaultBooking.orderSn || ''}</span>
            </p>
            {bookingData.unionFlag === 1 ? null : (
              <p style={{ marginTop: 5 }} className={Styles.BillStatue}>
                <Tag color={orderStatusColor[bookingData.orderStatus]}>
                  {global.ORDER_STATUS[bookingData.orderStatus || 0]}
                </Tag>
              </p>
            )}
            {/* <Divider /> */}
          </div>
          {this.renderRemainingAmount(bookingData.orderSn ? bookingData : defaultBooking, 0)}
          <Table
            bordered
            columns={columns}
            dataSource={houseDataList}
            pagination={page}
            onChange={this.handlePageChange}
          />
          {/* {this.renderBillAmount(room, key)}  */}
          <Divider />
        </div>
      </div>
    );
    // return (
    //   <div className={Styles.BillPage}>
    //     {roomReservationData.map((room, key) => (
    //       <div className={Styles.BillPageBlock} key={room.orderId}>
    //         <div className={Styles.BillPageHd}>
    //           {this.renderDropDown()}
    //           <p style={{ marginTop: 6, marginLeft: 20 }} className={Styles.BillCode}>
    //             房单号: <span>{room.orderSn}</span>
    //           </p>
    //           <p style={{ marginTop: 5 }} className={Styles.BillStatue}>
    //             <Tag color="#87d068">{global.ORDER_STATUS[room.orderStatus]}</Tag>
    //           </p>
    //           {/* <Divider /> */}
    //         </div>
    //         {this.renderRemainingAmount(room, key)}
    //         <Table bordered columns={columns} dataSource={room.postingList} pagination={false} />
    //         {/* {this.renderBillAmount(room, key)}  */}
    //         <Divider />
    //       </div>
    //     ))}
    //   </div>
    // );
  };

  renderBillInfo = () => {
    const {
      order: { roomReservationData },
    } = this.props;
    return roomReservationData && roomReservationData.length !== 0
      ? this.renderNotEmptyBill()
      : this.renderEmptyBill();
  };

  render() {
    return this.renderBillInfo();
  }
}
