import React, { PureComponent } from 'react';
import { connect } from 'dva';
import { Card, Form, Row, Col, Input, Button, message, Select, Modal, Radio } from 'antd';
import moment from 'moment';
import ButtonAuthorized from '@/components/ButtonAuthorized';
import { WORDANDNUM } from '@/constant/reg';
import {
  CIDSourceList,
  CIDReasonList,
  GuestSituationList,
  CIDProcessResultList,
  PayoutList,
} from '../config/preventCID';
import styles from '../Supplement.less';

const FormItem = Form.Item;
const Option = Select.Option;
const { TextArea } = Input;
const RadioGroup = Radio.Group;

const formItemLayout = {
  labelCol: { span: 8 },
  wrapperCol: { span: 16 },
};

@Form.create()
@connect(({ preventCID }) => ({ preventCID }))
export default class CidHandleModel extends PureComponent {
  constructor(props) {
    super(props);
    const { visible } = props;
    this.state = { visible, isAdd: false };
  }

  static getDerivedStateFromProps(nextProps, prevState) {
    const { visible, bookingId, preventCID } = nextProps;
    const isAdd = preventCID.CIDQueryResult === null;

    if (visible !== prevState.visible) {
      return { ...nextProps, isAdd };
    }

    return null;
  }

  componentDidUpdate(prevProps) {
    const { form, preventCID } = this.props;
    // 模态框从隐藏到展示
    if (!prevProps.visible && this.props.visible) {
      // 发现有CID处理数据时，回显数据
      preventCID.CIDQueryResult && form.setFieldsValue(preventCID.CIDQueryResult[0]);
    }
  }

  handleCancel = () => {
    const visible = false;
    const { onCancel, form } = this.props;

    form.resetFields();
    this.setState({ visible });

    typeof onCancel === 'function' && onCancel(visible);
  };

  handleSubmit = () => {
    let { form, bookingId, dispatch } = this.props;
    const { isAdd } = this.state;
    form.validateFields().then(res => {
      let params = { ...res, bookingId };
      dispatch({
        type: 'preventCID/fetchCIDHandle',
        payload: params,
      })
        .then(r => {
          message.success(`${isAdd ? '提交' : '修改'}成功`);
          this.handleCancel();
        })
        .catch(e => {
          message.error(e.msg);
        });
    });
  };

  render() {
    const { visible, isAdd } = this.state;
    const { disabled, form, preventCID } = this.props;
    const { getFieldDecorator } = form;

    return (
      <Modal
        maskClosable={false}
        width={700}
        visible={visible}
        title={`${isAdd ? '记录' : '修改'}CID处理信息`}
        onCancel={this.handleCancel}
        footer={
          <div style={{ textAlign: 'right' }}>
            <Button onClick={this.handleCancel} style={{ marginRight: 10 }}>
              取消
            </Button>
            <ButtonAuthorized
              onClick={this.handleSubmit}
              code={isAdd ? 'CRS3.2.1.4' : 'CRS3.2.1.6'}
              type="primary"
            >
              {isAdd ? '提交' : '修改'}
            </ButtonAuthorized>
          </div>
        }
      >
        <Form>
          <Row>
            <Col span={12}>
              <FormItem label="CID来源" {...formItemLayout}>
                {getFieldDecorator('source', {
                  initialValue: undefined,
                  rules: [{ required: true, message: '请选择CID来源' }],
                })(
                  <Select placeholder="请选择">
                    {CIDSourceList.map(v => (
                      <Option value={v.value} key={v.value}>
                        {v.label}
                      </Option>
                    ))}
                  </Select>
                )}
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem label="CID原因" {...formItemLayout}>
                {getFieldDecorator('reason', {
                  initialValue: undefined,
                  rules: [{ required: true, message: '请选择CID原因' }],
                })(
                  <Select placeholder="请选择">
                    {CIDReasonList.map(v => (
                      <Option value={v.value} key={v.value}>
                        {v.label}
                      </Option>
                    ))}
                  </Select>
                )}
              </FormItem>

              <FormItem label="处理结果" {...formItemLayout}>
                {getFieldDecorator('processResult', {
                  initialValue: undefined,
                  rules: [{ required: true, message: '请选择处理结果' }],
                })(
                  <Select placeholder="请选择">
                    {CIDProcessResultList.map(v => (
                      <Option value={v.value} key={v.value}>
                        {v.label}
                      </Option>
                    ))}
                  </Select>
                )}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem label="客人情况" {...formItemLayout}>
                {getFieldDecorator('guestSituation', {
                  initialValue: undefined,
                  rules: [{ required: true, message: '请选择客人情况' }],
                })(<RadioGroup placeholder="请选择" options={GuestSituationList} />)}
              </FormItem>
              <FormItem label="赔付" {...formItemLayout}>
                {getFieldDecorator('needPayout', {
                  initialValue: undefined,
                  rules: [{ required: true, message: '请选择是否需要赔付' }],
                })(<RadioGroup placeholder="请选择" options={PayoutList} />)}
              </FormItem>
            </Col>
          </Row>
          <FormItem
            label="转移后的订单"
            help="如果转移到附近OYO酒店请填写转移订单号"
            labelCol={{ span: 4 }}
            wrapperCol={{ span: 10 }}
          >
            {getFieldDecorator('transferBookingSn', {
              initialValue: undefined,
              rules: [{ pattern: WORDANDNUM, message: '单号不正确' }],
            })(<Input placeholder="请输入" />)}
          </FormItem>
          <FormItem label="备注" labelCol={{ span: 4 }} wrapperCol={{ span: 20 }}>
            {getFieldDecorator('remark', {
              initialValue: undefined,
              rules: [{ max: 500, message: '备注不可超过500字' }],
            })(<TextArea placeholder="请输入备注" rows={3} />)}
          </FormItem>
        </Form>
      </Modal>
    );
  }
}
