import React, { PureComponent } from 'react';
import { connect } from 'dva';
import {
  Card,
  Form,
  Row,
  Col,
  Input,
  Button,
  message,
  Select,
  Modal,
  Radio,
  InputNumber,
} from 'antd';
import moment from 'moment';
import { WORDANDNUM } from '@/constant/reg';
import {
  VOCSourceList,
  CIDComplaintList,
  VOCResultList,
  ResponsibleList,
} from '../config/preventCID';
import { checkFloatNumber } from '@/utils/validate';
import styles from '../Supplement.less';

const FormItem = Form.Item;
const Option = Select.Option;
const { TextArea } = Input;
const RadioGroup = Radio.Group;
const payoutNumValidator = (rules, value, callback) => {
  if (!checkFloatNumber(value)) {
    callback('赔付金额至多两位小数');
  } else if (value < 0 || value > 9999) {
    callback('赔付金额只能为0～9999');
  }
  callback();
};

const formItemLayout = {
  labelCol: { span: 8 },
  wrapperCol: { span: 16 },
};

@Form.create()
@connect(({ preventCID }) => ({ preventCID }))
export default class VOCHandleModel extends PureComponent {
  constructor(props) {
    super(props);
    const { visible } = props;
    this.state = { visible };
  }

  static getDerivedStateFromProps(nextProps, prevState) {
    const { visible, bookingId } = nextProps;

    if (visible !== prevState.visible) {
      return { ...nextProps };
    }

    return null;
  }

  handleCancel = () => {
    const visible = false;
    const { onCancel, form } = this.props;

    form.resetFields();
    this.setState({ visible });

    typeof onCancel === 'function' && onCancel(visible);
  };

  handleSubmit = () => {
    let { form, bookingId, dispatch } = this.props;
    form.validateFields().then(res => {
      let params = { ...res, bookingId };
      dispatch({
        type: 'preventCID/fetchVOCHandle',
        payload: params,
      })
        .then(r => {
          message.success('提交成功');
          this.handleCancel();
        })
        .catch(e => {
          message.error(e.msg);
        });
    });
  };

  render() {
    const { visible } = this.state;
    const { disabled, form } = this.props;
    const { getFieldDecorator } = form;

    return (
      <Modal
        maskClosable={false}
        width={700}
        visible={visible}
        title="记录订单赔付信息"
        onCancel={this.handleCancel}
        onOk={this.handleSubmit}
        okText="提交"
      >
        <Form>
          <Row gutter={20}>
            <Col span={12}>
              <FormItem label="赔付来源" {...formItemLayout}>
                {getFieldDecorator('payOutSource', {
                  initialValue: undefined,
                  rules: [{ required: true, message: '请选择赔付来源' }],
                })(
                  <Select placeholder="请选择">
                    {VOCSourceList.map(v => (
                      <Option value={v.value} key={v.value}>
                        {v.label}
                      </Option>
                    ))}
                  </Select>
                )}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem label="投诉事项" {...formItemLayout}>
                {getFieldDecorator('complaint', {
                  initialValue: undefined,
                  rules: [{ required: true, message: '请选择投诉事项!' }],
                })(
                  <Select placeholder="请选择">
                    {CIDComplaintList.map(v => (
                      <Option value={v.value} key={v.value}>
                        {v.label}
                      </Option>
                    ))}
                  </Select>
                )}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem label="投诉详情" labelCol={{ span: 4 }} wrapperCol={{ span: 20 }}>
                {getFieldDecorator('compaintDetail', {
                  initialValue: undefined,
                  rules: [
                    { required: true, message: '请输入投诉详情' },
                    { max: 500, message: '投诉详情不可超过500字' },
                  ],
                })(<TextArea placeholder="请输入" rows={3} />)}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem label="核实确认人" {...formItemLayout}>
                {getFieldDecorator('confirmer', {
                  initialValue: undefined,
                  rules: [
                    { required: true, message: '请输入核实确认人' },
                    { max: 50, message: '核实确认人不可超过50字' },
                  ],
                })(<Input placeholder="请输入" />)}
              </FormItem>
              <FormItem label="责任方" {...formItemLayout}>
                {getFieldDecorator('responsibleParty', {
                  initialValue: undefined,
                  rules: [{ required: true, message: '请选择责任方' }],
                })(
                  <Select placeholder="请选择">
                    {ResponsibleList.map(v => (
                      <Option value={v.value} key={v.value}>
                        {v.label}
                      </Option>
                    ))}
                  </Select>
                )}
              </FormItem>
              <FormItem label="赔付金额" {...formItemLayout}>
                {getFieldDecorator('payoutAmount', {
                  initialValue: undefined,
                  rules: [
                    { required: true, message: '请输入赔付金额' },
                    { validator: payoutNumValidator, message: '赔付金额至多两位小数，最大值9999' },
                  ],
                })(<Input addonBefore="¥" placeholder="请输入" type="number" />)}
              </FormItem>
              <FormItem label="开户人姓名" {...formItemLayout}>
                {getFieldDecorator('accountantName', {
                  initialValue: undefined,
                  rules: [
                    { required: true, message: '请输入开户人姓名' },
                    { max: 50, message: '开户人姓名不可超过50字' },
                  ],
                })(<Input placeholder="请输入" />)}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem label="处理结果" {...formItemLayout}>
                {getFieldDecorator('processResult', {
                  initialValue: undefined,
                  rules: [{ required: true, message: '请选择处理结果' }],
                })(
                  <Select placeholder="请选择">
                    {VOCResultList.map(v => (
                      <Option value={v.value} key={v.value}>
                        {v.label}
                      </Option>
                    ))}
                  </Select>
                )}
              </FormItem>
              <FormItem label="责任人" {...formItemLayout}>
                {getFieldDecorator('responsibleUser', {
                  initialValue: undefined,
                  rules: [
                    { required: true, message: '请输入责任人' },
                    { max: 50, message: '责任人不可超过50字' },
                  ],
                })(<Input placeholder="请输入" />)}
              </FormItem>
              <FormItem label="开户银行" {...formItemLayout}>
                {getFieldDecorator('accountBankName', {
                  initialValue: undefined,
                  rules: [
                    { required: true, message: '请输入开户银行' },
                    { max: 50, message: '开户银行不可超过50字' },
                  ],
                })(<Input placeholder="请输入" />)}
              </FormItem>
              <FormItem label="收款银行账号" {...formItemLayout}>
                {getFieldDecorator('payoutAccount', {
                  initialValue: undefined,
                  rules: [
                    { required: true, message: '请输入收款银行账号' },
                    { max: 50, message: '收款银行账号不可超过50字' },
                  ],
                })(<Input placeholder="请输入" />)}
              </FormItem>
            </Col>
          </Row>
        </Form>
      </Modal>
    );
  }
}
