import React, { PureComponent } from 'react';
import DateRules from './components/allDayRoom/DateRules';
import PriceChange from './components/allDayRoom/PriceChange';
import GuestSourceRule from './components/defaultRoom/GuestSourceRule';
import LeastReservationRule from './components/defaultRoom/LeastReservationRule';
import PayRule from './components/defaultRoom/PayRule';
import CancelRule from './components/defaultRoom/CancelRule';
import InventoryRule from './components/defaultRoom/InventoryRule';
import GuaranteeRule from './components/defaultRoom/GuaranteeRule';

/**
 * 校验可预定日期时间规则的限制时间段
 * @param  {Number} time 时间点
 * @return {Boolean} 是否通过校验
 */
function testTime(time) {
  if (time === undefined) return false;

  const stringTime = `${time}`;

  // 不足6位的时间直接pass
  if (stringTime.length < 6) return false;

  const timeArr = stringTime
    .replace(/(\d{2})(\d{2})(\d{2})/, ($0, $1, $2, $3) => `${$1},${$2},${$3}`)
    .split(',');
  const h = Number(timeArr[0]); // 小时
  const m = Number(timeArr[1]); // 分钟
  const s = Number(timeArr[2]); // 秒

  // 小时为24时 分钟与秒只能为0
  if (h === 24 && m + s > 0) return false;
  // 时间区间的校验
  if (h < 0 || h > 24 || m < 0 || m > 60 || s < 0 || s > 60) return false;

  return true;
}

export default class AllDayRoom extends PureComponent {
  render() {
    return (
      <>
        <DateRules />
        <PriceChange />
        <GuestSourceRule />
        <LeastReservationRule />
        <PayRule />
        <CancelRule />
        <InventoryRule />
        <GuaranteeRule />
      </>
    );
  }

  /**
   * 上传前所有全天房特有的字段的校验
   * @param  {Object} strategy model层的strategy内所有数据
   * @return {Boolean} state   最终校验的成果
   * @return {String}  msg     报错的内容
   * @return {String}  input   需要聚焦的input
   */
  static validate(strategy) {
    if (strategy.isUseChangePrice) {
      const {
        type,
        stablePrice,
        dynamicPriceValue,
        dynamicPriceType,
        rateCustomizable,
        customizeRateType,
        rangeFrom,
        rangeTo,
      } = strategy.changePriceRule;

      const { reservationDateRule } = strategy;

      // 使用限制时间段的情况
      if (reservationDateRule.isUseTimeLimit) {
        const timeLimitList = reservationDateRule.timeLimitList;

        for (let i = 0; i < timeLimitList.length; i++) {
          let { startTime, endTime } = timeLimitList[i];

          if (!testTime(startTime)) {
            return {
              state: false,
              msg: '限制时间段格式有误',
              input: `reservationDateRule-${i}-startTime`,
            };
          }

          if (!testTime(endTime)) {
            return {
              state: false,
              msg: '限制时间段格式有误',
              input: `reservationDateRule-${i}-endTime`,
            };
          }

          if (Number(startTime) > Number(endTime)) {
            return {
              state: false,
              msg: '限制时间段开始时间需早于结束时间',
              input: `reservationDateRule-${i}-startTime`,
            };
          }
        }
      }

      if (type === 0 && (stablePrice === null || stablePrice === undefined || stablePrice === '')) {
        return {
          state: false,
          msg: '请填写正确的固定价格',
          input: 'priceChangeStablePrice',
        };
      }

      if (
        type === 1 &&
        (dynamicPriceValue === null || dynamicPriceValue === undefined || dynamicPriceValue === '')
      ) {
        // this.scroll('priceChangeWrapper');
        return {
          state: false,
          msg: '请填写正确的浮动数据',
          input: `priceChangeDynamic-${dynamicPriceType}`,
        };
      }

      // 开启自定义价格
      if (rateCustomizable && customizeRateType === 1) {
        if (Number(rangeFrom) < 1) {
          return {
            state: false,
            msg: '范围在1-999',
            input: `customPriceRange-f`,
          };
        }

        if (Number(rangeTo) > 999) {
          return {
            state: false,
            msg: '范围在1-999',
            input: `customPriceRange-t`,
          };
        }
      }

      return {
        state: true,
      };
    }

    return {
      state: true,
    };
  }

  static getSaveDate(strategy, formatDate) {
    let {
      rateCodeName,
      description,
      status,
      checkinType,
      reservationDateRule,
      applyDateRule,
      applyDateRule2,
      changePriceRule,
      guestSourceRule,
      leastReservationRule,
      payRule,
      cancelRule,
      inventoryRule,
      guaranteeRule,
      isUseChangePrice,
      isUseLeastReservation,
    } = strategy;

    // 将分散的model内key值转入rules
    const payload = {
      rateCodeName,
      description,
      status,
      checkinType,
      rules: {
        reservationDateRule: { ...reservationDateRule }, // 复制一个全新的引用对象, 用于之后操作而不影响原本model
        applyDateRule: { ...applyDateRule }, // 同上
        guestSourceRule: { ...guestSourceRule },
        payRule,
        cancelRule,
        inventoryRule,
        guaranteeRule,
      },
    };

    if (guestSourceRule.guestSource.includes(0)) {
      payload.rules.guestSourceRule.guestSource = [0];
    }

    // 进行上传前的数据处理
    // 如下为可使用日期时间规则的两个大模块数据处理
    [reservationDateRule, applyDateRule].forEach(item => {
      // 前面小方块未被勾选的情况下,后端要求不传值
      if (!item.isUseWeekly) delete payload.rules[item.name].weekly;
      if (!item.isUseTimeLimit) {
        delete payload.rules[item.name].timeLimitList;
      }
      if (!item.isUseExceptDate) delete payload.rules[item.name].exceptDate;

      // 有选择固定日期的情况下 对moment数据进行转为8位数的参数要求
      if (item.regularDate.startDate) {
        payload.rules[item.name].regularDate = {
          startDate: formatDate(payload.rules[item.name].regularDate.startDate),
          endDate: formatDate(payload.rules[item.name].regularDate.endDate),
        };
      }
    });

    delete payload.rules.applyDateRule.timeLimitList;

    // 可使用起订规则switch打开时 加入leastReservationRule
    if (isUseLeastReservation) {
      payload.rules.leastReservationRule = leastReservationRule;
    }

    // 使用变价规则switch打开时
    if (isUseChangePrice) {
      payload.rules.changePriceRule = changePriceRule; // 可使用起订规则
      payload.rules.changePriceRule.stablePrice = Number(payload.rules.changePriceRule.stablePrice);
      payload.rules.changePriceRule.dynamicPriceValue = Number(
        payload.rules.changePriceRule.dynamicPriceValue
      );
    }

    return payload;
  }
}
