import moment, { isDate } from 'moment';
import React, { Component } from 'react';
import { parse, stringify } from 'qs';

export function fixedZero(val) {
  return val * 1 < 10 ? `0${val}` : val;
}

/**
 * @method
 * @param {moment} current 目标对象
 * @returns {boolean}
 * @desc 获取不可使用日期，返回true表示昨天及其以前不可使用日期
 */
export function disabledDate(current) {
  const today = moment().endOf('day');
  const yestoday = today.subtract(1, 'days').endOf('day');
  return current && current < yestoday;
}

export function getTimeDistance(type) {
  const now = new Date();
  const oneDay = 1000 * 60 * 60 * 24;

  if (type === 'today') {
    now.setHours(0);
    now.setMinutes(0);
    now.setSeconds(0);
    return [moment(now), moment(now.getTime() + (oneDay - 1000))];
  }

  if (type === 'week') {
    let day = now.getDay();
    now.setHours(0);
    now.setMinutes(0);
    now.setSeconds(0);

    if (day === 0) {
      day = 6;
    } else {
      day -= 1;
    }

    const beginTime = now.getTime() - day * oneDay;

    return [moment(beginTime), moment(beginTime + (7 * oneDay - 1000))];
  }

  if (type === 'month') {
    const year = now.getFullYear();
    const month = now.getMonth();
    const nextDate = moment(now).add(1, 'months');
    const nextYear = nextDate.year();
    const nextMonth = nextDate.month();

    return [
      moment(`${year}-${fixedZero(month + 1)}-01 00:00:00`),
      moment(moment(`${nextYear}-${fixedZero(nextMonth + 1)}-01 00:00:00`).valueOf() - 1000),
    ];
  }

  const year = now.getFullYear();
  return [moment(`${year}-01-01 00:00:00`), moment(`${year}-12-31 23:59:59`)];
}

export function getPlainNode(nodeList, parentPath = '') {
  const arr = [];
  nodeList.forEach(node => {
    const item = node;
    item.path = `${parentPath}/${item.path || ''}`.replace(/\/+/g, '/');
    item.exact = true;
    if (item.children && !item.component) {
      arr.push(...getPlainNode(item.children, item.path));
    } else {
      if (item.children && item.component) {
        item.exact = false;
      }
      arr.push(item);
    }
  });
  return arr;
}

function accMul(arg1, arg2) {
  let m = 0;
  const s1 = arg1.toString();
  const s2 = arg2.toString();
  m += s1.split('.').length > 1 ? s1.split('.')[1].length : 0;
  m += s2.split('.').length > 1 ? s2.split('.')[1].length : 0;
  return (Number(s1.replace('.', '')) * Number(s2.replace('.', ''))) / 10 ** m;
}

export function digitUppercase(n) {
  const fraction = ['角', '分'];
  const digit = ['零', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖'];
  const unit = [['元', '万', '亿'], ['', '拾', '佰', '仟', '万']];
  let num = Math.abs(n);
  let s = '';
  fraction.forEach((item, index) => {
    s += (digit[Math.floor(accMul(num, 10 * 10 ** index)) % 10] + item).replace(/零./, '');
  });
  s = s || '整';
  num = Math.floor(num);
  for (let i = 0; i < unit[0].length && num > 0; i += 1) {
    let p = '';
    for (let j = 0; j < unit[1].length && num > 0; j += 1) {
      p = digit[num % 10] + unit[1][j] + p;
      num = Math.floor(num / 10);
    }
    s = p.replace(/(零.)*零$/, '').replace(/^$/, '零') + unit[0][i] + s;
  }

  return s
    .replace(/(零.)*零元/, '元')
    .replace(/(零.)+/g, '零')
    .replace(/^整$/, '零元整');
}

function getRelation(str1, str2) {
  if (str1 === str2) {
    console.warn('Two path are equal!'); // eslint-disable-line
  }
  const arr1 = str1.split('/');
  const arr2 = str2.split('/');
  if (arr2.every((item, index) => item === arr1[index])) {
    return 1;
  }
  if (arr1.every((item, index) => item === arr2[index])) {
    return 2;
  }
  return 3;
}

function getRenderArr(routes) {
  let renderArr = [];
  renderArr.push(routes[0]);
  for (let i = 1; i < routes.length; i += 1) {
    // 去重
    renderArr = renderArr.filter(item => getRelation(item, routes[i]) !== 1);
    // 是否包含
    const isAdd = renderArr.every(item => getRelation(item, routes[i]) === 3);
    if (isAdd) {
      renderArr.push(routes[i]);
    }
  }
  return renderArr;
}

/**
 * Get router routing configuration
 * { path:{name,...param}}=>Array<{name,path ...param}>
 * @param {string} path
 * @param {routerData} routerData
 */
export function getRoutes(path, routerData) {
  let routes = Object.keys(routerData).filter(
    routePath => routePath.indexOf(path) === 0 && routePath !== path
  );
  // Replace path to '' eg. path='user' /user/name => name
  routes = routes.map(item => item.replace(path, ''));
  // Get the route to be rendered to remove the deep rendering
  const renderArr = getRenderArr(routes);
  // Conversion and stitching parameters
  const renderRoutes = renderArr.map(item => {
    const exact = !routes.some(route => route !== item && getRelation(route, item) === 1);
    return {
      exact,
      ...routerData[`${path}${item}`],
      key: `${path}${item}`,
      path: `${path}${item}`,
    };
  });
  return renderRoutes;
}

export function getPageQuery() {
  return parse(window.location.href.split('?')[1]);
}

export function getQueryPath(path = '', query = {}) {
  const search = stringify(query);
  if (search.length) {
    return `${path}?${search}`;
  }
  return path;
}

/* eslint no-useless-escape:0 */
const reg = /(((^https?:(?:\/\/)?)(?:[-;:&=\+\$,\w]+@)?[A-Za-z0-9.-]+(?::\d+)?|(?:www.|[-;:&=\+\$,\w]+@)[A-Za-z0-9.-]+)((?:\/[\+~%\/.\w-_]*)?\??(?:[-\+=&;%@.\w_]*)#?(?:[\w]*))?)$/;

export function isUrl(path) {
  return reg.test(path);
}

export function formatWan(val) {
  const v = val * 1;
  if (!v || Number.isNaN(v)) return '';

  let result = val;
  if (val > 10000) {
    result = Math.floor(val / 10000);
    result = (
      <span>
        {result}
        <span
          styles={{
            position: 'relative',
            top: -2,
            fontSize: 14,
            fontStyle: 'normal',
            lineHeight: 20,
            marginLeft: 2,
          }}
        >
          万
        </span>
      </span>
    );
  }
  return result;
}

export function typeCheck(types, source) {
  if (!types) {
    throw new Error('typeCheck need types pram');
  }
  let res = '';
  switch (types) {
    case 'isStr':
      res = Object.prototype.toString.call(source) === '[object String]';
      break;
    case 'isNum':
      res = Object.prototype.toString.call(source) === '[object Number]';
      break;
    case 'isBoolean':
      res = Object.prototype.toString.call(source) === '[object Boolean]';
      break;
    case 'isSym':
      res = Object.prototype.toString.call(source) === '[object Symbol]';
      break;

    case 'isUndefined':
      res = Object.prototype.toString.call(source) === '[object Undefined]';
      break;
    case 'isNull':
      res = Object.prototype.toString.call(source) === '[object Null]';
      break;
    case 'isArray':
      res = Object.prototype.toString.call(source) === '[object Array]';
      break;

    case 'isRegExp':
      res = Object.prototype.toString.call(source) === '[object RegExp]';
      break;

    case 'isError':
      res = Object.prototype.toString.call(source) === '[object Error]';
      break;

    case 'isDate':
      res = Object.prototype.toString.call(source) === '[object Date]';
      break;

    case 'isObj':
      res = Object.prototype.toString.call(source) === '[object Object]';
      break;

    default:
      throw new Error(`'typeCheck' funtion do not support types：${types}`);
  }
  return res;
}

export function isAntdPro() {
  return window.location.hostname === 'preview.pro.ant.design';
}

export function getUUID() {
  let s4 = () =>
    Math.floor((1 + Math.random()) * 0x10000)
      .toString(16)
      .substring(1);
  return `${s4() + s4()}-${s4()}-${s4()}-${s4()}-${s4() + s4() + s4()}`;
}

/**
 * 返回比较数组中相同的一部分
 * @param {String} compareKey
 * @param {Array} someArray
 * @param {Array} AllArray
 */
export function findSameItemInTwoArray(compareKey, someArray, AllArray) {
  if (compareKey === undefined && typeof compareKey !== 'string') return;
  if (!Array.isArray(someArray)) return;
  if (!Array.isArray(AllArray)) return;

  const key = compareKey;
  const some = someArray;
  const all = AllArray;
  let sameArray = [];

  if (some.length === 0) return sameArray;

  for (let i = 0; i < all.length; i++) {
    let allItem = all[i];
    let allKey = allItem[key];
    let isExist = false;
    for (let j = 0; j < some.length; j++) {
      let someItem = some[j];

      if (allKey === someItem) {
        isExist = true;

        break;
      }
    }

    if (isExist) {
      sameArray.push(allItem.name);
    }
  }
  return sameArray;
}

/**
 * 计算含中文字符串的长度
 * @param {String} val
 */
export function getByteLen(val) {
  let len = 0;
  for (let i = 0; i < val.length; i++) {
    let a = val.charAt(i);
    if (a.match(/[\u4e00-\u9fa5]/gi) != null) {
      len += 2;
    } else {
      len += 1;
    }
  }
  return len;
}

/**
 * @method
 * @param obj 过滤的对象
 * @param val 要过滤的值
 * @returns {object}
 * @desc 扩展omit的用途，删除对象中值等于自定义的key
 */
export function omitValue(obj, val = ['', undefined, null, NaN]) {
  if (!typeCheck('isObj', obj) && !typeCheck('isArray', val)) {
    return obj;
  }
  const omitObj = { ...obj };
  Object.keys(obj).forEach(keyItem => {
    val.forEach(valItem => {
      if (Object.is(obj[keyItem], valItem)) {
        delete omitObj[keyItem];
      }
    });
  });
  return omitObj;
}

/**
 * @method
 * @param {s} s 需要转化的数字或数字字符串
 * @param {n} n 小数点控制
 * @returns {number, string}
 * @desc 金额格式转化
 */
export function formatMoney(s, n) {
  let isFalse = false;
  n = n > 0 && n <= 20 ? n : 2;
  s = `${parseFloat(`${s}`.replace(/[^\d\.-]/g, '')).toFixed(n)}`;
  if (s.indexOf('-') !== -1) {
    s = s.split('-')[1];
    isFalse = true;
  }
  let l = s
    .split('.')[0]
    .split('')
    .reverse();
  let r = s.split('.')[1];
  let t = '';
  let w = '';
  for (let i = 0; i < l.length; i++) {
    t += l[i] + ((i + 1) % 3 === 0 && i + 1 !== l.length ? ',' : '');
  }
  if (isFalse) {
    w = `-${t
      .split('')
      .reverse()
      .join('')}.${r}`;
  } else {
    w = `${t
      .split('')
      .reverse()
      .join('')}.${r}`;
  }
  return w;
}

export function isReactComponentInstance(value) {
  return value instanceof Component;
}

/**
 * @description 获取加密数据
 * @param str->源数据 st->起始值 num->加密位数
 * @returns str->加工数据
 */
export function getEncryptionStr(str = '', st = 6, num = 8) {
  if (typeof str !== 'string' || typeof st !== 'number' || typeof num !== 'number') {
    return str;
  }

  const regExp = new RegExp(`(.{${st}})(.{0,${num}})`);

  return str.replace(regExp, ($0, $1, $2, $3) => {
    return $1 + '*'.repeat($2.length);
  });
}

/* 获取header参数 */
export function getHeaderParams(name, header) {
  const hReg = new RegExp(`(^|; ?)${name}=([^;]*)(;|$)`, 'i');
  const nameVal = header.match(hReg);
  if (nameVal != null) return unescape(nameVal[2]);
  return '';
}
